open Asm_types
open Tac_types


(* print the input sat *)
let print_sat sat =
        let lsate = NameMap.fold (fun k v l -> v :: l) sat [] in
        let rec print_lsate lsate =
                if lsate == [] then (print_endline "Sat Table Complete")
                else
                        begin
                        let sate = List.hd lsate in
                        let n_str = sate.sat_name in
                        let t_str =
                               (match sate.sat_type with
                                   Sa_tmp -> "tmp"
                                 | Sa_var -> "var"
                                 | Sa_invld -> "invld") in
                        let r_str = 
                               (match sate.sat_reg with
                                   Eax -> "eax"
                                 | Ebx -> "ebx"
                                 | Ecx -> "ecx"
                                 | Edx -> "edx"
                                 | Reg_none -> "none") in
                        let l_str = 
                               (match sate.sat_loc with
                                   Sa_loc_stack -> "stack"
                                 | Sa_loc_data -> "data"
                                 | Sa_loc_invld -> "invld") in
                        let o_str = (string_of_int sate.sat_soff) in
                        let nu_str = (string_of_int sate.sat_nextuse) in
                        print_endline ("\tNAME: " ^ n_str ^ " TYPE: " ^ t_str ^
                                " REG: " ^ r_str ^ " LOC: " ^ l_str ^
                                " OFFSET: " ^ o_str ^ " NEXTUSE: " ^
                                nu_str);
                        print_lsate (List.tl lsate)
                        end
        in
        let _ = print_endline ("PRINTING SYMBOL ALIAS TABLE:") in
        let _ = (print_lsate lsate) in 
        print_endline ("FINISHED PRINTING SYMBOL ALIAS TABLE")



let string_of_reg_status s =
        match s with
           Rs_free -> "free"
         | Rs_used -> "used"

let string_of_reg r =
        match r with
           Eax -> "eax"
         | Ebx -> "ebx"
         | Ecx -> "ecx"
         | Edx -> "edx"
         | Reg_none -> "none"


let debug_print_rt rt =
       print_endline ("REGISTER TABLE (RT) DEBUG PRINT:" ^
                "\n\tEAX: " ^ (fst rt.rt_eax) ^
                "   STATUS: " ^ (string_of_reg_status (snd rt.rt_eax)) ^
                "\n\tEBX: " ^ (fst rt.rt_ebx) ^
                "   STATUS: " ^ (string_of_reg_status (snd rt.rt_ebx)) ^ 
                "\n\tECX: " ^ (fst rt.rt_ecx) ^
                "   STATUS: " ^ (string_of_reg_status (snd rt.rt_ecx)) ^
                "\n\tEDX: " ^ (fst rt.rt_edx) ^
                "   STATUS: " ^ (string_of_reg_status (snd rt.rt_edx)) ^
                "\n\tNEXT EVICT: " ^ (string_of_reg rt.rt_nextevict))
