open Tac_types


module IntMap = Map.Make(struct
        type t = int
        let compare x y = Pervasives.compare x y
        end)

type asmgen_env = {
        age_implabel : int;
        age_soff : int;         (* stack offset *)
        age_doff : int;         (* data segment offset *)
        age_nlocals : int       (* number of locals in a function *)
        }

type register_resource = Eax | Ebx | Ecx | Edx | Reg_none
type symbol_alias_type = Sa_tmp | Sa_var | Sa_invld
type symbol_alias_location = Sa_loc_stack | Sa_loc_data | Sa_loc_invld

type symbol_alias_element = {
        sat_name : string;
        sat_type : symbol_alias_type;
        sat_size : int;
        sat_reg : register_resource;
        sat_loc : symbol_alias_location;
        sat_soff : int;
        sat_nextuse : int
        }

let invld_sat_symb = { 
        sat_name = "invld";
        sat_type = Sa_invld;
        sat_size = 0;
        sat_reg = Reg_none;
        sat_loc = Sa_loc_invld;
        sat_soff = 0;
        sat_nextuse = 0
        }

type register_status = Rs_free | Rs_used

type register_table = {
        rt_eax : string * register_status;
        rt_ebx : string * register_status;
        rt_ecx : string * register_status;
        rt_edx : string * register_status;
        rt_nextevict : register_resource
        }

type temporary_table = {
        tt_base : int;          (* base of temporaries with respect to ss:[ebp] *)
        tt_free : int list      (* free temp stack spots (not including base) *)
        }
