open Ast
open Tacgen
open Tac_display
open Asmgen
open Printf

let print = false

let _ =
  (* Make sure correct number of arguments is given *)
  if (Array.length Sys.argv) != 3 then
          raise (Failure("Num args: " ^ (string_of_int (Array.length Sys.argv)) ^
                " Expected 3."))
  else
  let filename = Sys.argv.(1) in
  let rcbseed = Sys.argv.(2) in
  let _ = print_endline ("Compiling File: " ^ filename) in
  let _ = print_endline ("RCB Seed Value: " ^ rcbseed) in

  (* Get the seed value and input channel *)
  let rcbseed_int = int_of_string rcbseed in
  let ic = open_in filename in

  (* Scan the file *)
  print_endline "Starting parsing...";
  let lexbuf = Lexing.from_channel ic in
  (* Parse the scanned tokens *)
  print_endline "Finished lexbuf definition...";
  let program = Parser.program Scanner.token lexbuf in
  print_endline "Finished parser program...";
  (* Create intermediate three-address code *)
  let tac_prog =
          Tacgen.tac_create_prog program rcbseed_int in
  let _ = Tac_display.print_tac_program tac_prog in
  (* Create the assembly listing *)
  let lasm = Asmgen.generate_assembly_program tac_prog in
  let lasm = List.rev lasm in
  let _ = List.map (fun x -> (print_endline x)) lasm in
  
  (* Write the results to an output assembly file *)
  let outfile = String.sub filename 0 ((String.length filename)-4) in
  let outfile = (outfile ^ ".asm") in
  let _ = print_endline ("Writing Assembly File: " ^ outfile) in

  let oc = open_out outfile in
  let _ = List.map (fun x -> (output_string oc x; output_char oc '\n')) lasm in
  let _ = close_out oc in
  1


