%{ 

open Ast

%}

%token SEMI LPAREN RPAREN LBRACE RBRACE LBRACKET RBRACKET COMMA
%token PLUS MINUS TIMES DIVIDE RDIVIDE XOR OR AND NOT
%token ASSIGN
%token EQ NEQ GT GEQ LT LEQ
%token IF ELSE FOR WHILE
%token RET
%token VOID CHAR ORD1 ORD2 ORD4 ARRAY
%token RCB TREGION
%token EOF
%token <int> LITERAL
%token <string> ID

%nonassoc NOELSE
%nonassoc ELSE

%right ASSIGN              /*  lowest precedence */
%left EQ NEQ LT LEQ GT GEQ
%left XOR OR AND NOT
%left PLUS MINUS
%left TIMES DIVIDE RDIVIDE
%nonassoc UMINUS           /*  highest precedence */

%start program
%type <Ast.program> program

%%

/* top level program */
program:
  /* nothing */      { [], [], [], [], [], [] }
| program vdecl      { $2 :: (fun (x, _, _, _, _, _) -> x) $1,
                       (fun (_, x, _, _, _, _) -> x) $1,
                       (fun (_, _, x, _, _, _) -> x) $1,
                       (fun (_, _, _, x, _, _) -> x) $1,
                       (fun (_, _, _, _, x, _) -> x) $1,
                       (fun (_, _, _, _, _, x) -> x) $1
                     }
| program fdecl      { (fun (x, _, _, _, _, _) -> x) $1,
                       $2 :: ((fun (_, x, _, _, _, _) -> x) $1), 
                       (fun (_, _, x, _, _, _) -> x) $1,
                       (fun (_, _, _, x, _, _) -> x) $1,
                       (fun (_, _, _, _, x, _) -> x) $1,
                       (fun (_, _, _, _, _, x) -> x) $1
                     }
| program rcbdecl    { (fun (x, _, _, _, _, _) -> x) $1,
                       (fun (_, x, _, _, _, _) -> x) $1, 
                       $2 :: ((fun (_, _, x, _, _, _) -> x) $1),
                       (fun (_, _, _, x, _, _) -> x) $1,
                       (fun (_, _, _, _, x, _) -> x) $1,
                       (fun (_, _, _, _, _, x) -> x) $1 
                     }
| program tregdecl   { (fun (x, _, _, _, _, _) -> x) $1,
                       (fun (_, x, _, _, _, _) -> x) $1, 
                       (fun (_, _, x, _, _, _) -> x) $1,
                       $2 :: ((fun (_, _, _, x, _, _) -> x) $1),
                       (fun (_, _, _, _, x, _) -> x) $1,
                       (fun (_, _, _, _, _, x) -> x) $1
                     }
| program fdef       { (fun (x, _, _, _, _, _) -> x) $1,
                       (fun (_, x, _, _, _, _) -> x) $1, 
                       (fun (_, _, x, _, _, _) -> x) $1,
                       (fun (_, _, _, x, _, _) -> x) $1,
                       $2 :: ((fun (_, _, _, _, x, _) -> x) $1),
                       (fun (_, _, _, _, _, x) -> x) $1
                     }
| program rcbdef     { (fun (x, _, _, _, _, _) -> x) $1,
                       (fun (_, x, _, _, _, _) -> x) $1, 
                       (fun (_, _, x, _, _, _) -> x) $1,
                       (fun (_, _, _, x, _, _) -> x) $1,
                       (fun (_, _, _, _, x, _) -> x) $1,
                       $2 :: ((fun (_, _, _, _, _, x) -> x) $1)
                     }


/* variable declaration */
vdecl:
  data_type ID SEMI                                      { Uninitvardecl($1, $2) } 
| ARRAY ID LBRACKET LITERAL RBRACKET SEMI                { Uninitarraydecl($2, $4) }
| data_type ID ASSIGN LITERAL SEMI                       { Initvardecl($1, $2, $4) } 
| ARRAY ID LBRACKET LITERAL RBRACKET ASSIGN LITERAL SEMI { Initarraydecl($2, $4, $7) }



/* function declaration */
fdecl:
  data_type ID LPAREN opt_paramsdecl RPAREN SEMI { 
          { fdecl_name = $2;
            fdecl_params = List.rev $4;
            fdecl_rtype = $1 }
        }

/* optional parameter declarations */ 
opt_paramsdecl:
  /* nothing */   { [] }
| paramsdecl_list { $1 }

/* list of parameter declarations */
paramsdecl_list:
  paramdecl { [$1] }
| paramsdecl_list COMMA paramdecl { $3 :: $1 }

/* parameter declaration */
paramdecl:
  data_type ID                           { Pdecl($1, $2); }
| ARRAY ID LBRACKET LITERAL RBRACKET     { Pdecla($2, $4); }

/* random code block declaration */
rcbdecl:
  RCB ID SEMI { $2 }

/* region of memory being tested declaration */
tregdecl:
  TREGION ID SEMI { $2 }


/* a function definition */
fdef:
  data_type ID LPAREN opt_paramsdecl RPAREN LBRACE opt_vdecl_list opt_stmt_list RBRACE  { 
          { fdef_name = $2;
            fdef_params = $4;
            fdef_vdecls = $7;
            fdef_body =  $8;
            fdef_rtype = $1 }
  }

/* a random code block definition */
rcbdef:
  RCB ID LBRACE opt_vdecl_list opt_stmt_list RBRACE {
          { rcbdef_name = $2;
            rcbdef_vdecls = $4;
            rcbdef_body = $5 }
  }

/* optional parameters */
opt_params:
        /* nothing */ { [] }
| params_list { List.rev $1 }

/* a list of paramters */
params_list:
        param { [$1] }
| params_list COMMA param { $3 :: $1 }

/* a single parameter */
param:
  ID { Id($1) }
| LITERAL { Literal($1) }


opt_stmt_list:
  /* nothing */ { [] }
| stmt_list     { List.rev $1 }

opt_vdecl_list:
  /* nothing */ { [] }
| vdecl_list    { List.rev $1 }

stmt_list:
  stmt { [$1] }
| stmt_list stmt { $2 :: $1 }

vdecl_list:
  vdecl { [$1] }
| vdecl_list vdecl { $2 :: $1 }


/* statements */
stmt:
  expr SEMI                                       { Expr($1) }
| RET expr SEMI                                   { Return($2) }
| LBRACE opt_stmt_list RBRACE                     { Block($2) }
| IF LPAREN expr RPAREN stmt %prec NOELSE         { If($3, $5, Nostmt) } 
| IF LPAREN expr RPAREN stmt ELSE stmt            { If($3, $5, $7) }
| FOR LPAREN expr SEMI expr SEMI expr RPAREN stmt { For($3, $5, $7, $9) }
| WHILE LPAREN expr RPAREN stmt                   { While($3, $5) }
| LBRACKET rcb_list RBRACKET SEMI                 { Rcbcall($2) }

/* expressions */
expr:
  LITERAL                                         { Literal($1) }
| ID                                              { Id($1) }
| ID LBRACKET expr RBRACKET data_type             { Array($1, $3, $5) }
| expr PLUS expr                                  { Binop($1, Add, $3) }
| expr MINUS expr                                 { Binop($1, Sub, $3) }
| expr TIMES expr                                 { Binop($1, Mult, $3) }
| expr DIVIDE expr                                { Binop($1, Div, $3) }
| expr RDIVIDE expr                               { Binop($1, Rdiv, $3) }
| expr XOR expr                                   { Binop($1, Xor, $3) }
| expr OR expr                                    { Binop($1, Or, $3) }
| expr AND expr                                   { Binop($1, And, $3) }
| expr EQ expr                                    { Binop($1, Equal, $3) }
| expr NEQ expr                                   { Binop($1, Neq, $3) }
| expr LT expr                                    { Binop($1, Less, $3) }
| expr LEQ expr                                   { Binop($1, Leq, $3) }
| expr GT expr                                    { Binop($1, Greater, $3) }
| expr GEQ expr                                   { Binop($1, Geq, $3) }
| MINUS expr %prec UMINUS                         { Uniop(Uminus, $2) }
| NOT expr                                        { Uniop(Not, $2) }
| ID ASSIGN expr                                  { Assign($1, $3) }
| ID LBRACKET expr RBRACKET data_type ASSIGN expr { Arrayassign($1, $3, $5, $7) }
| ID LPAREN opt_params RPAREN                     { Call($1, $3) }
| LPAREN expr RPAREN                              { Paren($2) } 


/* a list of random code block identifiers */
rcb_list:
  rcb_list COMMA rcb  {  $3 :: $1 }
| rcb                 { [$1] }

/* a single random code block
 *   this contains both the identifier for the random code block 
 *   (potentially) being inserted, as well as a numerical value
 *   representing the "weight" of it being inserted.
 *
 *   this has format:  (num)id -> (23)my_rcb_1
 */
rcb:
  LPAREN LITERAL RPAREN ID   { ($2, $4) } 

/* the data types - character and ordinals */
data_type: 
  CHAR { Char }
| ORD1 { Ord1 }  
| ORD2 { Ord2 }
| ORD4 { Ord4 }
| VOID { Void }
