(* Header Section *)
{ open Parser }

(* Definitions Section *)
let digit = ['0' - '9']
let digit_h = ['0' - '9' 'a' - 'f' 'A' - 'F']
let number = ("0x")digit_h+ | digit+
let letter = ['a' - 'z' 'A' - 'Z' '_']
let id = letter ( letter | digit )*

(* Rules Section *)
rule token = parse
| [' ' '\t' '\n' '\r'] { token lexbuf }
| "/*"            { comment 0 lexbuf }
| '('             { LPAREN }
| ')'             { RPAREN }
| '{'             { LBRACE }
| '}'             { RBRACE }
| '['             { LBRACKET }
| ']'             { RBRACKET }
| ';'             { SEMI }
| ','             { COMMA }
| '+'             { PLUS }
| '-'             { MINUS }
| '*'             { TIMES }
| '/'             { DIVIDE }
| '%'             { RDIVIDE }
| '^'             { XOR }
| '|'             { OR }
| '&'             { AND }
| '!'             { NOT }
| '='             { ASSIGN }
| "=="            { EQ }
| "!="            { NEQ }
| '<'             { LT }
| "<="            { LEQ }
| '>'             { GT }
| ">="            { GEQ }
| "if"            { IF }
| "else"          { ELSE }
| "for"           { FOR }
| "while"         { WHILE }
| "return"        { RET }
| "void"          { VOID }
| "char"          { CHAR }
| "ord1"          { ORD1 }
| "ord2"          { ORD2 }
| "ord4"          { ORD4 }
| "array"         { ARRAY }
| "rcode"         { RCB }
| "#test_region"  { TREGION }
| number as lxm   { LITERAL(int_of_string lxm) }
| id as lxm       { ID(lxm) }
| eof             { EOF }
| _ as invd       { raise (Failure("Illegal character " ^ Char.escaped invd)) }

(* Support for Multi-Level Comments *)
and comment level = parse
| "*/"            { if level = 0 then token lexbuf
                    else comment (level - 1) lexbuf
                  }
| "/*"            { comment (level + 1) lexbuf }
| _               { comment level lexbuf       }
| eof             { print_endline "File ends within comment";
                    raise End_of_file
                  }
