open Ast

module NameMap = Map.Make(struct
        type t = string
        let compare x y = Pervasives.compare x y
        end)

type symbol_type = Svar | Srcb | Sfunc | Sarray

type symbol = {
        symb_name : string;
        symb_stype : symbol_type;
        symb_dtype : data_type;
        symb_offset : int;
        symb_init : int;
        symb_size : int;
        symb_isparam : bool;
        symb_paramnum : int;
        symb_paramsizes : int list
        }

type dest_src_type = Dvar | Darray | Dimm | Dtmp 

type tac_dest_src = {
        tds_type : dest_src_type;
        tds_name : string;
        tds_value : int;
        tds_size : int
        }

type tac_op = Tadd | Tsub | Tmul | Tdiv | Trdiv | Txor | Tor | Tand | Teq | Tne
            | Tlt | Tgt | Tle | Tge | Tuminus | Tnot | Tjmp | Tjmpe | Tjmpne
            | Tjmpgt | Tjmplt | Tjmpge | Tjmple | Tlabel | Tarray | Tasn | Taasn
            | Tcmp | Tret | Tparam | Tcall | Tevictall | Tespadd | Tevicteax 
            | Tevictebx | Tevictecx | Tevictedx

let get_tac_op_from_binop op =
        match op with
            Add -> Tadd
          | Sub -> Tsub
          | Mult -> Tmul
          | Div -> Tdiv
          | Rdiv -> Trdiv 
          | Xor -> Txor
          | Or -> Tor
          | And -> Tand
          | Equal -> Teq
          | Neq -> Tne
          | Less -> Tlt
          | Leq -> Tle
          | Greater -> Tgt
          | Geq -> Tge

let get_tac_op_from_uniop op =
        match op with
            Uminus -> Tuminus
          | Not -> Tnot

type tac_element = {
        to_op : tac_op;
        to_dest : tac_dest_src;
        to_src1 : tac_dest_src;
        to_src2 : tac_dest_src
        }

type tac_rcb = {
        rcb_symb : symbol;
        rcb_globals : symbol NameMap.t;
        rcb_locals : symbol NameMap.t;
        rcb_ltac : tac_element list
        }

type tac_func = {
        func_symb : symbol;
        func_globals : symbol NameMap.t;
        func_locals : symbol NameMap.t;
        func_ltac : tac_element list
        }

type tac_program = {
        prog_globals : symbol NameMap.t;
        prog_lrcb : tac_rcb list;
        prog_lfunc : tac_func list
        }

type tac_env = {
        te_label : int;         (* current label  *)
        te_trslt : int;         (* tmp result num *)
        te_seed : int;          (* seed for RCB elements *)
        te_group : int          (* group that tac element belongs too *)
        }

let dest_src_invld = {
        tds_type = Dvar;
        tds_name = "invld";
        tds_value = 0;
        tds_size = 0 
        }

let dest_src_zero = {
        tds_type = Dimm;
        tds_name = "invld";
        tds_value = 0;
        tds_size = 1
        }
